/*
  Program that changes light intesity of an RGB strip based on sensor values from an RGB sensor
  -in case a certain color is detected, the strip will display that color
  -To properly calibrate the color detection, you have to first run an RGB test with your TV
   to define the threshold value for each color
  -in case the measured data doesn't correspond to a certain color, the RGB values will display an ambient white corresponding
  to the average of all the color values.
  -Changes in color/light values are done smoothly using linear change with a user defined delay
  
*/

#include <Wire.h>
#include <SparkFun_APDS9960.h>

/*Pin Definitions*/
//LED strip PWM pins
#define pin_blue 3
#define pin_red 5
#define pin_green 6

//User input potentiometer pins
#define pin_pot_delay A1
#define pin_pot_intensity A2

/*Constant values*/
const int COLOR_THRESHOLD = 2; //Based on RGB values from test (set to 0 to display only white)

//user input value ranges
const int DELAY_MIN = 100;            //Minimum time you can set for smooth light changes (msec)
const int DELAY_MAX = 5000;           //Maximum time you can set for smooth light changes (msec) 
const long int INTENSITY_MIN = 1000;  //Minimum multiplier value you can set for brightness (x1000)
const long int INTENSITY_MAX = 50000; //Maximum multiplier value you can set for brightness (x1000)

//Minimum PWM value for LED brightness (this prevents the strip from flashing when screen brightness is too low)
const double MIN_PWM_VALUE = 1;

/*Global Variables*/
//APDS9960 Variables
SparkFun_APDS9960 apds = SparkFun_APDS9960();
uint16_t ambient_light = 0;
uint16_t red_light = 0;
uint16_t green_light = 0;
uint16_t blue_light = 0;

//Variables to detect a change in sensor readings
uint16_t ambient_light_previous = 0;
uint16_t red_light_previous = 0;
uint16_t green_light_previous = 0;
uint16_t blue_light_previous = 0;

//Variables for user defined brightness
double intensity;
double intensity_previous = 0.0;

void setup() {
  
  // Initialize Serial port
  //Uncomment Serial functions for debugging*/
  /*Serial.begin(9600);
  Serial.println();
  Serial.println(F("--------------------------------"));
  Serial.println(F("SparkFun APDS-9960 - ColorSensor"));
  Serial.println(F("--------------------------------"));*/
  
  // Initialize APDS-9960 (configure I2C and initial value_reds)
  if ( apds.init() ) {
    //Serial.println(F("APDS-9960 initialization complete"));
  } else {
    //Serial.println(F("Something went wrong during APDS-9960 init!"));
  }
  
  // Start running the APDS-9960 light sensor (no interrupts)
  if ( apds.enableLightSensor(false) ) {
    //Serial.println(F("Light sensor is now running"));
  } else {
    //Serial.println(F("Something went wrong during light sensor init!"));
  }
  
  // Wait for initialization and calibration to finish
  delay(500);
}

/*Filter variables and constants*/

double value_red;
double previous_value_red = 0.0;
double slope_red;                   //slope of value_red change
double pwm_value_red;

double value_green;
double previous_value_green = 0.0;
double slope_green;                 //slope of value_green change
double pwm_value_green;

double value_blue;
double previous_value_blue = 0.0;
double slope_blue;                  //slope of value_blue change
double pwm_value_blue;


/*timer values*/
unsigned long int time_value_change_red = 0;  //time of value change
unsigned long int timer_value_change_red = 0; //time since value change

unsigned long int time_value_change_green = 0;  //time of value change
unsigned long int timer_value_change_green = 0; //time since value change

unsigned long int time_value_change_blue = 0;  //time of value change
unsigned long int timer_value_change_blue = 0; //time since value change

long double timer_value_change_sec= 0;   //time since value change in seconds
double time_delay = 3.0;                 //time it will take for output to settle

unsigned int time_sample = 20; //filter sampling time (ms)
unsigned int timer_sample = 0;

/*potentiometer values*/
int read_delay;
int read_intensity;

void loop() {

  read_delay = analogRead(pin_pot_delay);
  read_intensity = analogRead(pin_pot_intensity);

  time_delay = map(read_delay, 0, 1023, DELAY_MIN, DELAY_MAX);
  intensity = map(read_intensity, 0, 1023, INTENSITY_MIN, INTENSITY_MAX);

  time_delay = time_delay/1000.0;
  intensity = intensity/1000.0;

   // Read the light levels (ambient, red, green, blue)
  if (  !apds.readAmbientLight(ambient_light) ||
        !apds.readRedLight(red_light) ||
        !apds.readGreenLight(green_light) ||
        !apds.readBlueLight(blue_light) ) {
    ///Serial.println("Error reading light value");
  }

  /*Color detection*/
  
  //Red
  if ((red_light >= COLOR_THRESHOLD) && (green_light < COLOR_THRESHOLD) && (blue_light < COLOR_THRESHOLD)) {
    green_light = blue_light = 0;
  }
  //Green
  else if ((red_light < COLOR_THRESHOLD) && (green_light >= COLOR_THRESHOLD) && (blue_light < COLOR_THRESHOLD)) {
    red_light = blue_light = 0;
  }
  //Blue
  else if ((red_light < COLOR_THRESHOLD) && (green_light < COLOR_THRESHOLD) && (blue_light >= COLOR_THRESHOLD)) {
    red_light = green_light = 0;
  }
  //Cyan
  else if ((red_light < COLOR_THRESHOLD) && (green_light >= COLOR_THRESHOLD) && (blue_light >= COLOR_THRESHOLD)) {
    red_light = 0;
  }
  //Magenta
  else if ((red_light >= COLOR_THRESHOLD) && (green_light < COLOR_THRESHOLD) && (blue_light >= COLOR_THRESHOLD)) {
    green_light = 0;
  }
  //Yellow
  else if ((red_light >= COLOR_THRESHOLD) && (green_light >= COLOR_THRESHOLD) && (blue_light < COLOR_THRESHOLD)) {
    blue_light = 0;
  }
  //Other (show white light)
  else {
    double average_light = (double(red_light) + double(green_light) + double(blue_light))/3.0;
    red_light = green_light = blue_light = average_light;
  }

  /*Detect color changes. Also detects change in intensity input value*/

  /*Red light change detection*/
  if(red_light != red_light_previous || intensity != intensity_previous) {
    intensity_previous = intensity;
    red_light_previous = red_light;
    previous_value_red = pwm_value_red;
    time_value_change_red = millis();
    value_red = red_light*intensity;
    if (value_red > 255)
      value_red = 255;
    else if (value_red < 0)
      value_red = 0;
    slope_red = (value_red - previous_value_red)/time_delay;
  }
  /*Green light change detection*/
  if(green_light != green_light_previous || intensity != intensity_previous) {
    intensity_previous = intensity;
    green_light_previous = green_light;
    previous_value_green = pwm_value_green;
    time_value_change_green = millis();
    value_green = green_light*intensity;
    if (value_green > 255)
      value_green = 255;
    else if (value_green < 0)
      value_green = 0;
    slope_green = (value_green - previous_value_green)/time_delay;
  }

  /*Blue light change detection*/
  if(blue_light != blue_light_previous || intensity != intensity_previous) {
    intensity_previous = intensity;
    blue_light_previous = blue_light;
    previous_value_blue = pwm_value_blue;
    time_value_change_blue = millis();
    value_blue = blue_light*intensity;
    if (value_blue > 255)
      value_blue = 255;
    else if (value_blue < 0)
      value_blue = 0;
    slope_blue = (value_blue - previous_value_blue)/time_delay;
  }

  if (millis() - timer_sample >= time_sample) {
    timer_sample = millis();
    
    /*Red light change*/
    timer_value_change_red = millis() - time_value_change_red;
    timer_value_change_sec= double(timer_value_change_red)/1000;
    if (timer_value_change_sec < time_delay) {
      pwm_value_red = previous_value_red + slope_red*timer_value_change_sec;
    }
    else {
      pwm_value_red = value_red;
    }
    
    //bounds
    if (pwm_value_red < MIN_PWM_VALUE)
      pwm_value_red = MIN_PWM_VALUE;
    if (pwm_value_red > 255)
      pwm_value_red = 255;

    /*Green light change*/
    timer_value_change_green = millis() - time_value_change_green;
    timer_value_change_sec= double(timer_value_change_green)/1000;
    if (timer_value_change_sec < time_delay) {
      pwm_value_green = previous_value_green + slope_green*timer_value_change_sec;
    }
    else {
      pwm_value_green = value_green;
    }
    
    //bounds
    if (pwm_value_green < MIN_PWM_VALUE)
      pwm_value_green = MIN_PWM_VALUE;
    if (pwm_value_green > 255)
      pwm_value_green = 255;
    
    /*blue light change*/
    timer_value_change_blue = millis() - time_value_change_blue;
    timer_value_change_sec= double(timer_value_change_blue)/1000;
    if (timer_value_change_sec < time_delay) {
      pwm_value_blue = previous_value_blue + slope_blue*timer_value_change_sec;
    }
    else {
      pwm_value_blue = value_blue;
    }
    
    //bounds
    if (pwm_value_blue < MIN_PWM_VALUE)
      pwm_value_blue = MIN_PWM_VALUE;
    if (pwm_value_blue > 255)
      pwm_value_blue = 255;
    
    //Print out reading and PWM values
    /*Serial.print(red_light);
    Serial.print(",");
    Serial.print(green_light);
    Serial.print(",");
    Serial.print(blue_light);
    Serial.print(",");
    Serial.print(pwm_value_red);
    Serial.print(",");
    Serial.print(pwm_value_green);
    Serial.print(",");
    Serial.println(pwm_value_blue);*/

    analogWrite(pin_red, pwm_value_red);
    analogWrite(pin_green, pwm_value_green);
    analogWrite(pin_blue, pwm_value_blue);
  }
}

